@@ -1,8 +1,11 @@
 package Lingua::YaTeA::LexiconItem;
 use strict;
+use warnings;
 
 our $counter =0;
 
+our $VERSION=$Lingua::YaTeA::VERSION;
+
 sub new
 {
     my ($class,$form) = @_;
