@@ -1,9 +1,12 @@
 package Lingua::YaTeA::MonolexicalTermCandidate;
 use strict;
+use warnings;
 use Lingua::YaTeA::TermCandidate;
 
 our @ISA = qw(Lingua::YaTeA::TermCandidate);
 
+our $VERSION=$Lingua::YaTeA::VERSION;
+
 sub new
 {
     my ($class) = @_;
