@@ -1,8 +1,11 @@
 package Lingua::YaTeA::Occurrence;
 use strict;
+use warnings;
 
 our $counter = 0;
 
+our $VERSION=$Lingua::YaTeA::VERSION;
+
 sub new
 {
     my ($class,) = @_;
