@@ -1,9 +1,13 @@
 package Lingua::YaTeA::PatternLeaf;
 use strict;
+use warnings;
+
 use Lingua::YaTeA::Edge;
 
 our @ISA = qw(Lingua::YaTeA::Edge);
 
+our $VERSION=$Lingua::YaTeA::VERSION;
+
 sub new
 {
     my ($class,$tag,$father) = @_;
