@@ -1,8 +1,11 @@
 package Lingua::YaTeA::WordOccurrence;
 use strict;
+use warnings;
 use Lingua::YaTeA::ChunkingDataSet;
 use UNIVERSAL qw(isa);
 
+our $VERSION=$Lingua::YaTeA::VERSION;
+
 sub new
 {
     my ($class,$form) = @_;
