@@ -2,7 +2,9 @@
 
 package Lingua::YaTeA::XMLEntities;
 use strict;
+use warnings;
 
+our $VERSION=$Lingua::YaTeA::VERSION;
 
 sub encode
 {
